# -*- coding: utf-8 -*-
"""1d mimetic polynomials in the reference domain [-1,1].

© mathischeap.com
"""
import numpy as np
from scipy.sparse import csr_matrix
from quadrature import lobatto


def node_polynomials(nodes, xi):
    """Compute the lagrange polynomials built at ``nodes`` evaluated at
    ``xi``.

    :param nodes: The nodes on which the Lagrange polynomials are built.
    :param xi: The points to evaluate the Lagrange polynomials.
    :return: A 2d numpy.array whose 0-dimension refers to the
        polynomials and 1-dimension refers to the values evaluated at
        ``xi``.
    :example:

    >>> nodes, _ = lobatto(3)
    >>> x = np.linspace(-1, 1, 50)
    >>> lp = node_polynomials(nodes, x)
    >>> lp[0,:] # doctest: +ELLIPSIS
    array([ 1.        ,  0.88167345,  0.77142177,  0.66898996,...
    """
    p = np.size(nodes)
    basis = np.ones((p, np.size(xi)))
    for i in range(p):
        for j in range(p):
            if i != j:
                basis[i, :] *= (xi - nodes[j]) / (nodes[i] - nodes[j])
    return basis

def edge_polynomials(nodes, xi):
    """Compute the edge polynomials built at ``nodes`` evaluated at
    ``xi``.

    :param nodes: The nodes on which the edge polynomials are built.
    :param xi: The points to evaluate the edge polynomials.
    :return: A 2d numpy.array whose 0-dimension refers to the
        polynomials and 1-dimension refers to the values evaluated at
        ``xi``.
    :example:

    >>> nodes, _ = lobatto(4)
    >>> x = np.linspace(-1, 1, 50)
    >>> ep = edge_polynomials(nodes, x)
    >>> ep[2,:] # doctest: +ELLIPSIS
    array([ 0.91016418,  0.60853047,  0.34702611,  0.12374712, ...
    """
    p = np.size(nodes) - 1
    xi_xj = nodes.reshape(p + 1, 1) - nodes.reshape(1, p + 1)
    xi_xj[np.diag_indices(p + 1)] = 1
    c_i = np.prod(xi_xj, axis=1)
    c_i_div_cj = np.transpose(c_i.reshape(1, p+1) / c_i.reshape(p+1, 1))
    derivative = c_i_div_cj / xi_xj
    derivative[np.diag_indices(p + 1)] = 0
    derivative[np.diag_indices(p + 1)] = -np.sum(derivative, axis=1)
    polynomials = node_polynomials(nodes, xi)
    derivatives_poly = np.transpose(derivative) @ polynomials
    edge_poly = np.zeros((p, np.size(xi)))
    for i in range(p):
        for j in range(i + 1):
            edge_poly[i] -= derivatives_poly[j, :]
    return edge_poly

def incidence_matrix_E(N):
    """Compute the sparse incidence matrix.

    :param N: The degree of the nodes.
    :return: A scipy.sparse.csr_matrix matrix representing the
        incidence matrix.
    :example:

    >>> incidence_matrix_E(3).toarray()
    array([[-1,  1,  0,  0],
           [ 0, -1,  1,  0],
           [ 0,  0, -1,  1]])
    """
    data = [1 if i%2 == 1 else -1 for i in range(2*N)]
    indices = [i//2 for i in range(1, 2*N+1)]
    indptr = [2*i for i in range(N+1)]
    return csr_matrix((data, indices, indptr), shape=[N, N+1])

if __name__ == '__main__':
    import doctest
    doctest.testmod()